/*
 * This is based on
 * 
 * ESP8266 Web server with Web Socket to control an LED.
 *
 * The web server keeps all clients' LED status up to date and any client may
 * turn the LED on or off. Now to dim LED by slider.
 *
 * For example, clientA connects and turns the LED on. This changes the word
 * "LED" on the web page to the color red. When clientB connects, the word
 * "LED" will be red since the server knows the LED is on.  When clientB turns
 * the LED off, the word LED changes color to black on clientA and clientB web
 * pages.
 *
 * References:
 *
 * https://github.com/Links2004/arduinoWebSockets
 */


void webSocketEvent(uint8_t num, WStype_t type, uint8_t * payload, size_t length)
{
  Serial.printf("webSocketEvent(%d, %d, ...)\r\n", num, type);
  switch(type) {
    case WStype_DISCONNECTED:
      if (debug ) Serial.printf("[%u] Disconnected!\r\n", num);
      break;
    case WStype_CONNECTED:
      {
        IPAddress ip = webSocket.remoteIP(num);
        if (debug ) Serial.printf("[%u] Connected from %d.%d.%d.%d url: %s\r\n", num, ip[0], ip[1], ip[2], ip[3], payload);
        // Send the current Dimmvalue
        String dimmstring = dimmer + dimmvalue;
        int dimmstring_length = dimmstring.length() + 1;
        char sendstring[dimmstring_length];
        dimmstring.toCharArray(sendstring, dimmstring_length);
        webSocket.sendTXT(num, sendstring, strlen(sendstring));
      }
      break;
    case WStype_TEXT:
      {
        if (debug ) Serial.printf("[%u] get Text: %s\r\n", num, payload);
        String readstring = String((const char *)payload);
        int firstequal = readstring.indexOf('=') + 1;
        if (firstequal == 5) {
          long newdimmvalue = readstring.substring(5).toInt();
          if (debug ) Serial.printf("New dimmvalue [%d]\r\n", newdimmvalue);
          writeLED(newdimmvalue);
        }
        else {
          if (debug ) Serial.println("Unknown command");
        }
        // send data to all connected clients
        webSocket.broadcastTXT(payload, length);
      }  
      break;
    case WStype_BIN:
      if (debug ) Serial.printf("[%u] get binary length: %u\r\n", num, length);
      hexdump(payload, length);

      // echo data back to browser
      webSocket.sendBIN(num, payload, length);
      break;
    default:
      if (debug ) Serial.printf("Invalid WStype [%d]\r\n", type);
      break;
  }
}


